library ieee;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
library work;
use work.std_logic_arithext.all;


--datapath entity
entity CORE is
   port(
      message:in std_logic_vector(511 downto 0);
      Hashvalue:out std_logic_vector(15 downto 0);
      start:in std_logic;
      EoM:in std_logic;
      shift:out std_logic;
      rst_n : in std_logic;
      clk : in std_logic

   );
end CORE;


--signal declaration
architecture RTL of CORE is
signal Qint:std_logic_vector(1023 downto 0);
signal Qint_wire:std_logic_vector(1023 downto 0);
signal en_g:std_logic;
signal en_g_wire:std_logic;
signal en_d:std_logic;
signal en_d_wire:std_logic;
signal half:std_logic;
signal half_wire:std_logic;
signal rd:std_logic;
signal rd_wire:std_logic;
signal r_start:std_logic;
signal r_start_wire:std_logic;
signal r_shift:std_logic;
signal r_shift_wire:std_logic;
signal r_EoM:std_logic;
signal r_EoM_wire:std_logic;
signal counter:std_logic_vector(5 downto 0);
signal counter_wire:std_logic_vector(5 downto 0);
signal address:std_logic_vector(5 downto 0);
signal address_wire:std_logic_vector(5 downto 0);
signal output:std_logic_vector(15 downto 0);
signal output_wire:std_logic_vector(15 downto 0);
signal roundconstant:std_logic_vector(255 downto 0);
signal A:std_logic_vector(1023 downto 0);
signal s_Qin:std_logic_vector(1023 downto 0);
signal s_Qout:std_logic_vector(1023 downto 0);
signal B:std_logic_vector(1023 downto 0);
signal Qin:std_logic_vector(1023 downto 0);
signal Qout:std_logic_vector(1023 downto 0);
signal Q0:std_logic_vector(127 downto 0);
signal Q1:std_logic_vector(127 downto 0);
signal Q2:std_logic_vector(127 downto 0);
signal Q3:std_logic_vector(127 downto 0);
signal Q4:std_logic_vector(127 downto 0);
signal Q5:std_logic_vector(127 downto 0);
signal Q6:std_logic_vector(127 downto 0);
signal Q7:std_logic_vector(127 downto 0);
signal sig_9:std_logic_vector(5 downto 0);
signal sig_10:std_logic;
signal sig_11:std_logic_vector(5 downto 0);
signal sig_12:std_logic_vector(255 downto 0);
signal sig_13:std_logic_vector(383 downto 0);
signal sig_14:std_logic_vector(511 downto 0);
signal sig_15:std_logic_vector(639 downto 0);
signal sig_16:std_logic_vector(767 downto 0);
signal sig_17:std_logic_vector(895 downto 0);
signal sig_18:std_logic_vector(1023 downto 0);
signal sig_19:std_logic_vector(511 downto 0);
signal sig_20:std_logic_vector(1023 downto 0);
signal sig_21:std_logic_vector(511 downto 0);
signal sig_22:std_logic_vector(1023 downto 0);
signal sig_23:std_logic_vector(1023 downto 0);
signal Hashvalue_int:std_logic_vector(15 downto 0);
signal shift_int:std_logic;
signal sig_24:std_logic;
signal sig_25:std_logic;
signal sig_0:std_logic_vector(3 downto 0);
signal sig_1:std_logic_vector(127 downto 0);
signal sig_2:std_logic_vector(127 downto 0);
signal sig_3:std_logic_vector(127 downto 0);
signal sig_4:std_logic_vector(127 downto 0);
signal sig_5:std_logic_vector(127 downto 0);
signal sig_6:std_logic_vector(127 downto 0);
signal sig_7:std_logic_vector(127 downto 0);
signal sig_8:std_logic_vector(127 downto 0);


--component map declaration
component ROM_constant
   port(
      address:in std_logic_vector(5 downto 0);
      rd:in std_logic;
      oput:out std_logic_vector(255 downto 0);
      rst_n : in std_logic;
      clk : in std_logic
   );
end component;
component degrouping
   port(
      Q:in std_logic_vector(1023 downto 0);
      B:out std_logic_vector(1023 downto 0);
      enable:in std_logic;
      rst_n : in std_logic;
      clk : in std_logic
   );
end component;
component grouping
   port(
      A:in std_logic_vector(1023 downto 0);
      Q:out std_logic_vector(1023 downto 0);
      enable:in std_logic;
      rst_n : in std_logic;
      clk : in std_logic
   );
end component;
component round8
   port(
      Qin:in std_logic_vector(1023 downto 0);
      roundconstant:in std_logic_vector(255 downto 0);
      half:in std_logic;
      Qout:out std_logic_vector(1023 downto 0);
      rst_n : in std_logic;
      clk : in std_logic
   );
end component;
type STATE_TYPE is (s0,s1,s2,s3,s4,s5,s6,s7,s8,s9,s10);
signal STATE:STATE_TYPE;
type CONTROL is (in_valuedefshift0disable
, resetalldefshift1disable
, beforeE8read1shift0enableGdefdisable
, resetalldefdisable
, beforeE8read1enableGdefdisable
, E8startaddress1resetalldefdisable
, round35address0half0_1def
, round35address1counter1def
, round35enableDshift1half0_1def
, E8finalresetalldefdisable
, afterE8defdisable
, sendoutdefdisablecounter1
, disabledef
);
signal cmd : CONTROL;


begin


   --portmap
   label_ROM_constant : ROM_constant port map (
         address => address,
         rd => rd,
         oput => roundconstant,
         rst_n => rst_n,
         clk => clk
      );
   label_degrouping : degrouping port map (
         Q => s_Qout,
         B => B,
         enable => en_d,
         rst_n => rst_n,
         clk => clk
      );
   label_grouping : grouping port map (
         A => A,
         Q => s_Qin,
         enable => en_g,
         rst_n => rst_n,
         clk => clk
      );
   label_round8 : round8 port map (
         Qin => Qin,
         roundconstant => roundconstant,
         half => half,
         Qout => Qout,
         rst_n => rst_n,
         clk => clk
      );
   --register updates
   dpREG: process (clk,rst_n)
      begin
         if (rst_n = '0') then
            Qint <= (others=>'0');
            en_g <= '0';
            en_d <= '0';
            half <= '0';
            rd <= '0';
            r_start <= '0';
            r_shift <= '0';
            r_EoM <= '0';
            counter <= (others=>'0');
            address <= (others=>'0');
            output <= (others=>'0');
         elsif clk' event and clk = '1' then
            Qint <= Qint_wire;
            en_g <= en_g_wire;
            en_d <= en_d_wire;
            half <= half_wire;
            rd <= rd_wire;
            r_start <= r_start_wire;
            r_shift <= r_shift_wire;
            r_EoM <= r_EoM_wire;
            counter <= counter_wire;
            address <= address_wire;
            output <= output_wire;

         end if;
      end process dpREG;


   --combinational logics
   dpCMB: process (Qint,en_g,en_d,half,rd,r_start,r_shift,r_EoM,counter,address
,output,roundconstant,A,s_Qin,s_Qout,B,Qin,Qout,Q0,Q1
,Q2,Q3,Q4,Q5,Q6,Q7,sig_9,sig_10,sig_11,sig_12
,sig_13,sig_14,sig_15,sig_16,sig_17,sig_18,sig_19,sig_20,sig_21,sig_22
,sig_23,Hashvalue_int,shift_int,message,start,EoM,cmd,STATE)
      begin
         Qint_wire <= Qint;
         en_g_wire <= en_g;
         en_d_wire <= en_d;
         half_wire <= half;
         rd_wire <= rd;
         r_start_wire <= r_start;
         r_shift_wire <= r_shift;
         r_EoM_wire <= r_EoM;
         counter_wire <= counter;
         address_wire <= address;
         output_wire <= output;
         A <= (others=>'0');
         s_Qout <= (others=>'0');
         Qin <= (others=>'0');
         Q0 <= (others=>'0');
         Q1 <= (others=>'0');
         Q2 <= (others=>'0');
         Q3 <= (others=>'0');
         Q4 <= (others=>'0');
         Q5 <= (others=>'0');
         Q6 <= (others=>'0');
         Q7 <= (others=>'0');
         sig_9 <= (others=>'0');
         sig_10 <= '0';
         sig_11 <= (others=>'0');
         sig_12 <= (others=>'0');
         sig_13 <= (others=>'0');
         sig_14 <= (others=>'0');
         sig_15 <= (others=>'0');
         sig_16 <= (others=>'0');
         sig_17 <= (others=>'0');
         sig_18 <= (others=>'0');
         sig_19 <= (others=>'0');
         sig_20 <= (others=>'0');
         sig_21 <= (others=>'0');
         sig_22 <= (others=>'0');
         sig_23 <= (others=>'0');
         Hashvalue_int <= (others=>'0');
         shift_int <= '0';
         sig_0 <= "0000";
         sig_1 <= "11001001011010001011100011100010110001010011101001011001011011100100001001111110010001011110111100011101011110101110011011100101";
         sig_2 <= "01100001010001011011011111011001000001100111000100011111011110100010111111000111011000010111100000000110101010010010001000000001";
         sig_3 <= "01111011001010011001000111000001101110010001100100101001111000101100010000101011010011001110000110001100110001011010001011010110";
         sig_4 <= "01100010001000001011111011001010100100000001101101011101110111111101001110110010000001010110001110001110101001111010110001011111";
         sig_5 <= "00010100001111101000110010111010011011010011000100110001000001001011000011100111000000000101010010010000010100100111001001110001";
         sig_6 <= "01001100110011100011001000011110000001110101110111100101000100000001101110101000000000001110110011100010000000100101000101111000";
         sig_7 <= "10011111010101110111001001111001010111111101000100000100101001011111000010111000101101100011010000100101111101011011001000111000";
         sig_8 <= "00010110011100001111101000111110010111111001000001111111000101111110001010001111110000000110010011100111011010011010110010010000";
         Hashvalue <= (others=>'0');
         shift <= '0';



         case cmd is
            when in_valuedefshift0disable =>
               sig_12 <= Q0 & Q1;
               sig_13 <= sig_12 & Q2;
               sig_14 <= sig_13 & Q3;
               sig_15 <= sig_14 & Q4;
               sig_16 <= sig_15 & Q5;
               sig_17 <= sig_16 & Q6;
               sig_18 <= sig_17 & Q7;
               Qint_wire <= sig_18;
               Hashvalue <= Hashvalue_int;
               shift <= shift_int;
               Q0 <= sig_1;
               Q1 <= sig_2;
               Q2 <= sig_3;
               Q3 <= sig_4;
               Q4 <= sig_5;
               Q5 <= sig_6;
               Q6 <= sig_7;
               Q7 <= sig_8;
               Hashvalue_int <= output;
               shift_int <= r_shift;
               r_start_wire <= start;
               r_EoM_wire <= EoM;
               r_shift_wire <= '0';
               Qin <= conv_std_logic_vector(0,1024);
            when resetalldefshift1disable =>
               en_g_wire <= '0';
               en_d_wire <= '0';
               half_wire <= '0';
               counter_wire <= conv_std_logic_vector(0,6);
               output_wire <= conv_std_logic_vector(unsigned(sig_0),16);
               Hashvalue <= Hashvalue_int;
               shift <= shift_int;
               Q0 <= sig_1;
               Q1 <= sig_2;
               Q2 <= sig_3;
               Q3 <= sig_4;
               Q4 <= sig_5;
               Q5 <= sig_6;
               Q6 <= sig_7;
               Q7 <= sig_8;
               Hashvalue_int <= output;
               shift_int <= r_shift;
               r_start_wire <= start;
               r_EoM_wire <= EoM;
               r_shift_wire <= '1';
               Qin <= conv_std_logic_vector(0,1024);
            when beforeE8read1shift0enableGdefdisable =>
               sig_19 <= Qint(1023 downto 512) xor message;
               sig_20 <= sig_19 & Qint(511 downto 0);
               Qint_wire <= sig_20;
               rd_wire <= '1';
               r_shift_wire <= '0';
               en_g_wire <= '1';
               en_d_wire <= '0';
               Hashvalue <= Hashvalue_int;
               shift <= shift_int;
               Q0 <= sig_1;
               Q1 <= sig_2;
               Q2 <= sig_3;
               Q3 <= sig_4;
               Q4 <= sig_5;
               Q5 <= sig_6;
               Q6 <= sig_7;
               Q7 <= sig_8;
               Hashvalue_int <= output;
               shift_int <= r_shift;
               r_start_wire <= start;
               r_EoM_wire <= EoM;
               Qin <= conv_std_logic_vector(0,1024);
            when resetalldefdisable =>
               en_g_wire <= '0';
               en_d_wire <= '0';
               half_wire <= '0';
               counter_wire <= conv_std_logic_vector(0,6);
               output_wire <= conv_std_logic_vector(unsigned(sig_0),16);
               Hashvalue <= Hashvalue_int;
               shift <= shift_int;
               Q0 <= sig_1;
               Q1 <= sig_2;
               Q2 <= sig_3;
               Q3 <= sig_4;
               Q4 <= sig_5;
               Q5 <= sig_6;
               Q6 <= sig_7;
               Q7 <= sig_8;
               Hashvalue_int <= output;
               shift_int <= r_shift;
               r_start_wire <= start;
               r_EoM_wire <= EoM;
               Qin <= conv_std_logic_vector(0,1024);
            when beforeE8read1enableGdefdisable =>
               sig_19 <= Qint(1023 downto 512) xor message;
               sig_20 <= sig_19 & Qint(511 downto 0);
               Qint_wire <= sig_20;
               rd_wire <= '1';
               en_g_wire <= '1';
               en_d_wire <= '0';
               Hashvalue <= Hashvalue_int;
               shift <= shift_int;
               Q0 <= sig_1;
               Q1 <= sig_2;
               Q2 <= sig_3;
               Q3 <= sig_4;
               Q4 <= sig_5;
               Q5 <= sig_6;
               Q6 <= sig_7;
               Q7 <= sig_8;
               Hashvalue_int <= output;
               shift_int <= r_shift;
               r_start_wire <= start;
               r_EoM_wire <= EoM;
               Qin <= conv_std_logic_vector(0,1024);
            when E8startaddress1resetalldefdisable =>
               A <= Qint;
               Qint_wire <= s_Qin;
               sig_11 <= unsigned(address) + unsigned(conv_std_logic_vector(1,6));
               address_wire <= sig_11;
               rd_wire <= '1';
               en_g_wire <= '0';
               en_d_wire <= '0';
               half_wire <= '0';
               counter_wire <= conv_std_logic_vector(0,6);
               output_wire <= conv_std_logic_vector(unsigned(sig_0),16);
               Hashvalue <= Hashvalue_int;
               shift <= shift_int;
               Q0 <= sig_1;
               Q1 <= sig_2;
               Q2 <= sig_3;
               Q3 <= sig_4;
               Q4 <= sig_5;
               Q5 <= sig_6;
               Q6 <= sig_7;
               Q7 <= sig_8;
               Hashvalue_int <= output;
               shift_int <= r_shift;
               r_start_wire <= start;
               r_EoM_wire <= EoM;
               Qin <= conv_std_logic_vector(0,1024);
            when round35address0half0_1def =>
               Qin <= Qint;
               Qint_wire <= Qout;
               address_wire <= conv_std_logic_vector(0,6);
               rd_wire <= '0';
               sig_10 <=  not half;
               half_wire <= sig_10;
               Hashvalue <= Hashvalue_int;
               shift <= shift_int;
               Q0 <= sig_1;
               Q1 <= sig_2;
               Q2 <= sig_3;
               Q3 <= sig_4;
               Q4 <= sig_5;
               Q5 <= sig_6;
               Q6 <= sig_7;
               Q7 <= sig_8;
               Hashvalue_int <= output;
               shift_int <= r_shift;
               r_start_wire <= start;
               r_EoM_wire <= EoM;
            when round35address1counter1def =>
               Qin <= Qint;
               Qint_wire <= Qout;
               sig_11 <= unsigned(address) + unsigned(conv_std_logic_vector(1,6));
               address_wire <= sig_11;
               rd_wire <= '1';
               sig_9 <= unsigned(counter) + unsigned(conv_std_logic_vector(1,6));
               counter_wire <= sig_9;
               Hashvalue <= Hashvalue_int;
               shift <= shift_int;
               Q0 <= sig_1;
               Q1 <= sig_2;
               Q2 <= sig_3;
               Q3 <= sig_4;
               Q4 <= sig_5;
               Q5 <= sig_6;
               Q6 <= sig_7;
               Q7 <= sig_8;
               Hashvalue_int <= output;
               shift_int <= r_shift;
               r_start_wire <= start;
               r_EoM_wire <= EoM;
            when round35enableDshift1half0_1def =>
               Qin <= Qint;
               Qint_wire <= Qout;
               en_d_wire <= '1';
               en_g_wire <= '0';
               r_shift_wire <= '1';
               sig_10 <=  not half;
               half_wire <= sig_10;
               Hashvalue <= Hashvalue_int;
               shift <= shift_int;
               Q0 <= sig_1;
               Q1 <= sig_2;
               Q2 <= sig_3;
               Q3 <= sig_4;
               Q4 <= sig_5;
               Q5 <= sig_6;
               Q6 <= sig_7;
               Q7 <= sig_8;
               Hashvalue_int <= output;
               shift_int <= r_shift;
               r_start_wire <= start;
               r_EoM_wire <= EoM;
            when E8finalresetalldefdisable =>
               s_Qout <= Qint;
               Qint_wire <= B;
               en_g_wire <= '0';
               en_d_wire <= '0';
               half_wire <= '0';
               counter_wire <= conv_std_logic_vector(0,6);
               output_wire <= conv_std_logic_vector(unsigned(sig_0),16);
               Hashvalue <= Hashvalue_int;
               shift <= shift_int;
               Q0 <= sig_1;
               Q1 <= sig_2;
               Q2 <= sig_3;
               Q3 <= sig_4;
               Q4 <= sig_5;
               Q5 <= sig_6;
               Q6 <= sig_7;
               Q7 <= sig_8;
               Hashvalue_int <= output;
               shift_int <= r_shift;
               r_start_wire <= start;
               r_EoM_wire <= EoM;
               Qin <= conv_std_logic_vector(0,1024);
            when afterE8defdisable =>
               sig_21 <= Qint(511 downto 0) xor message;
               sig_22 <= Qint(1023 downto 512) & sig_21;
               Qint_wire <= sig_22;
               Hashvalue <= Hashvalue_int;
               shift <= shift_int;
               Q0 <= sig_1;
               Q1 <= sig_2;
               Q2 <= sig_3;
               Q3 <= sig_4;
               Q4 <= sig_5;
               Q5 <= sig_6;
               Q6 <= sig_7;
               Q7 <= sig_8;
               Hashvalue_int <= output;
               shift_int <= r_shift;
               r_start_wire <= start;
               r_EoM_wire <= EoM;
               Qin <= conv_std_logic_vector(0,1024);
            when sendoutdefdisablecounter1 =>
               sig_23 <= Qint(1007 downto 0) & Qint(1023 downto 1008);
               Qint_wire <= sig_23;
               output_wire <= Qint(255 downto 240);
               Hashvalue <= Hashvalue_int;
               shift <= shift_int;
               Q0 <= sig_1;
               Q1 <= sig_2;
               Q2 <= sig_3;
               Q3 <= sig_4;
               Q4 <= sig_5;
               Q5 <= sig_6;
               Q6 <= sig_7;
               Q7 <= sig_8;
               Hashvalue_int <= output;
               shift_int <= r_shift;
               r_start_wire <= start;
               r_EoM_wire <= EoM;
               Qin <= conv_std_logic_vector(0,1024);
               sig_9 <= unsigned(counter) + unsigned(conv_std_logic_vector(1,6));
               counter_wire <= sig_9;
            when disabledef =>
               Qin <= conv_std_logic_vector(0,1024);
               Hashvalue <= Hashvalue_int;
               shift <= shift_int;
               Q0 <= sig_1;
               Q1 <= sig_2;
               Q2 <= sig_3;
               Q3 <= sig_4;
               Q4 <= sig_5;
               Q5 <= sig_6;
               Q6 <= sig_7;
               Q7 <= sig_8;
               Hashvalue_int <= output;
               shift_int <= r_shift;
               r_start_wire <= start;
               r_EoM_wire <= EoM;
            when others=>
         end case;
      end process dpCMB;


   --controller reg
   fsmREG: process (clk,rst_n)
      begin
         if (rst_n = '0') then
            STATE <= s0;
         elsif clk' event and clk = '1' then
            STATE <= STATE;
            case STATE is
               when s0 => 
                  if (r_start = '1') then
                          STATE <= s2;
                  else
                          STATE <= s0;
                  end if;
               when s1 => 
                  if (r_start = '1') then
                          STATE <= s3;
                  else
                          STATE <= s1;
                  end if;
               when s2 => 
                       STATE <= s3;
               when s3 => 
                       STATE <= s4;
               when s4 => 
                  if (sig_24 = '1') then
                          STATE <= s5;
                  else
                          STATE <= s4;
                  end if;
               when s5 => 
                       STATE <= s6;
               when s6 => 
                       STATE <= s7;
               when s7 => 
                  if (r_EoM = '1') then
                          STATE <= s8;
                  else
                          STATE <= s1;
                  end if;
               when s8 => 
                  if (sig_25 = '1') then
                          STATE <= s0;
                  else
                          STATE <= s9;
                  end if;
               when s9 => 
                       STATE <= s10;
               when s10 => 
                       STATE <= s8;
               when others=>
            end case;
         end if;
      end process fsmREG;


   --controller cmb
   fsmCMB: process (Qint,en_g,en_d,half,rd,r_start,r_shift,r_EoM,counter,address
,output,roundconstant,A,s_Qin,s_Qout,B,Qin,Qout,Q0,Q1
,Q2,Q3,Q4,Q5,Q6,Q7,sig_9,sig_10,sig_11,sig_12
,sig_13,sig_14,sig_15,sig_16,sig_17,sig_18,sig_19,sig_20,sig_21,sig_22
,sig_23,Hashvalue_int,shift_int,sig_24,sig_25,message,start,EoM,cmd,STATE)
      begin
      sig_24 <= '0';
      sig_25 <= '0';
      if (unsigned(counter) = 34) then
         sig_24 <= '1';
      else
         sig_24 <= '0';
      end if;
      if (unsigned(counter) = 31) then
         sig_25 <= '1';
      else
         sig_25 <= '0';
      end if;
      cmd <= in_valuedefshift0disable;
      case STATE is
         when s0 => 
            if (r_start = '1') then
                    cmd <= in_valuedefshift0disable;
            else
                    cmd <= resetalldefshift1disable;
            end if;
         when s1 => 
            if (r_start = '1') then
                    cmd <= beforeE8read1shift0enableGdefdisable;
            else
                    cmd <= resetalldefdisable;
            end if;
         when s2 => 
                 cmd <= beforeE8read1enableGdefdisable;
         when s3 => 
                 cmd <= E8startaddress1resetalldefdisable;
         when s4 => 
            if (sig_24 = '1') then
                    cmd <= round35address0half0_1def;
            else
                    cmd <= round35address1counter1def;
            end if;
         when s5 => 
                 cmd <= round35enableDshift1half0_1def;
         when s6 => 
                 cmd <= E8finalresetalldefdisable;
         when s7 => 
            if (r_EoM = '1') then
                    cmd <= afterE8defdisable;
            else
                    cmd <= afterE8defdisable;
            end if;
         when s8 => 
            if (sig_25 = '1') then
                    cmd <= sendoutdefdisablecounter1;
            else
                    cmd <= sendoutdefdisablecounter1;
            end if;
         when s9 => 
                 cmd <= disabledef;
         when s10 => 
                 cmd <= disabledef;
         when others=>
         end case;
   end process fsmCMB;
end RTL;
